<?php

use JsonApi\Schemas\BlubberComment as Schema;
use JsonApi\Routes\Blubber\CommentsDelete;

require_once 'BlubberTestHelper.php';

/**
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
class BlubberCommentsDeleteTest extends \Codeception\Test\Unit
{
    use BlubberTestHelper;

    /**
     * @var \UnitTester
     */
    protected $tester;

    protected function _before()
    {
        \DBManager::getInstance()->setConnection('studip', $this->getModule('\\Helper\\StudipDb')->dbh);
    }

    protected function _after()
    {
    }

    // tests
    public function testCreateComment()
    {
        // given
        $credentials1 = $this->tester->getCredentialsForTestAutor();
        $credentials2 = $this->tester->getCredentialsForRoot();
        $thread = $this->createPublicBlubberThreadForUser($credentials2, 'Who knows Daskylos?');

        $num = \BlubberComment::countBySQL('1');
        $comment = $this->tester->withPHPLib(
            $credentials1,
            fn() => $this->createBlubberComment($credentials1, $thread, 'Autolykos knows him.')
        );
        $this->tester->assertEquals($num + 1, \BlubberComment::countBySQL('1'));

        $response = $this->deleteBlubberCommentJSONAPI($credentials2, $comment);
        $this->tester->assertSame(204, $response->getStatusCode());

        $this->tester->assertEquals($num, \BlubberComment::countBySQL('1'));
    }

    private function deleteBlubberCommentJSONAPI(array $credentials, \BlubberComment $comment)
    {
        return $this->tester->callJsonApi(
            CommentsDelete::class,
            $credentials,
            ['id' => $comment->id],
            fn($requestBuilder) => $requestBuilder->delete()
        );
    }
}
