<?php

use JsonApi\Routes\Blubber\Rel\Mentions;
use JsonApi\Schemas\User as UsersSchema;

require_once 'BlubberTestHelper.php';

class BlubberMentionsRelationshipTest extends \Codeception\Test\Unit
{
    use BlubberTestHelper;

    /**
     * @var \UnitTester
     */
    protected $tester;

    protected function _before()
    {
        \DBManager::getInstance()->setConnection('studip', $this->getModule('\\Helper\\StudipDb')->dbh);
    }

    protected function _after()
    {
    }

    // tests
    public function testFetchRelationship()
    {
        $credentials = $this->tester->getCredentialsForTestAutor();

        $thread = $this->createPrivateBlubberThreadForUser($credentials, [$credentials]);

        $response = $this->tester->callJsonApi(Mentions::class, $credentials, ['id' => $thread->id]);
        $this->tester->assertTrue($response->isSuccessfulDocument([200]));

        $document = $response->document();
        $this->tester->assertTrue($document->isResourceCollectionDocument());

        $resources = $document->primaryResources();
        $this->tester->assertCount(count($thread->mentions), $resources);
    }

    public function testAddRelationship()
    {
        $credentials = $this->tester->getCredentialsForTestAutor();

        $thread = $this->createPrivateBlubberThreadForUser($credentials, [$credentials]);
        $this->tester->assertCount(1, $thread->mentions);

        $body = $this->prepareValidBody([$this->tester->getCredentialsForTestDozent()]);
        $response = $this->tester->callJsonApi(
            Mentions::class,
            $credentials,
            ['id' => $thread->id],
            fn($requestBuilder) => $requestBuilder->create()->setJsonApiBody($body)
        );

        $this->tester->assertSame(204, $response->getStatusCode());
        $this->tester->assertCount(2, \BlubberThread::find($thread->id)->mentions);
    }

    public function testRemoveRelationship()
    {
        $credentials1 = $this->tester->getCredentialsForTestAutor();
        $credentials2 = $this->tester->getCredentialsForTestDozent();
        $credentials3 = $this->tester->getCredentialsForTestAdmin();

        $thread = $this->createPrivateBlubberThreadForUser($credentials1, [
            $credentials1,
            $credentials2,
            $credentials3,
        ]);
        $this->tester->assertCount(3, $thread->mentions);

        $body = $this->prepareValidBody([$credentials1]);
        $response = $this->tester->callJsonApi(
            Mentions::class,
            $credentials1,
            ['id' => $thread->id],
            fn($requestBuilder) => $requestBuilder->delete()->setJsonApiBody($body)
        );

        $this->tester->assertSame(204, $response->getStatusCode());
        $this->tester->assertCount(2, \BlubberThread::find($thread->id)->mentions);
    }

    private function prepareValidBody(array $users)
    {
        return ['data' => array_map(fn($user) => ['type' => UsersSchema::TYPE, 'id' => $user['id']], $users)];
    }
}
