<?php

use JsonApi\Routes\Blubber\ThreadsShow;
use JsonApi\Routes\Blubber\CommentsByThreadIndex;
use JsonApi\Routes\Blubber\CommentsShow;

/**
 * @SuppressWarnings(PHPMD.StaticAccess)
 * @SuppressWarnings(PHPMD.Superglobals)
 */
trait BlubberTestHelper
{
    private function createPublicBlubberThreadForUser(array $credentials, string $content = '')
    {
        $blubber = \BlubberThread::create([
            'context_type' => 'public',
            'context_id' => $credentials['id'],
            'content' => $content,
            'user_id' => $credentials['id'],
            'external_contact' => 0,
            'visible_in_stream' => 1,
            'commentable' => 1,
        ]);

        return \BlubberThread::find($blubber->id);
    }

    private function createPrivateBlubberThreadForUser(array $credentials, array $mentions, string $content = '')
    {
        $blubber = \BlubberThread::create([
            'context_type' => 'private',
            'context_id' => '',
            'content' => $content,
            'user_id' => $credentials['id'],
            'external_contact' => 0,
            'visible_in_stream' => 1,
            'commentable' => 1,
        ]);

        foreach ($mentions as $mention) {
            \BlubberMention::create(['thread_id' => $blubber->id, 'user_id' => $mention['id']]);
        }

        return \BlubberThread::find($blubber->id);
    }

    private function createCourseBlubberThreadForUser(array $credentials, string $courseId, string $content = '')
    {
        $blubber = \BlubberThread::create([
            'context_type' => 'course',
            'context_id' => $courseId,
            'content' => $content,
            'user_id' => $credentials['id'],
            'external_contact' => 0,
            'visible_in_stream' => 1,
            'commentable' => 1,
        ]);

        return \BlubberThread::find($blubber->id);
    }

    private function createInstituteBlubberThreadForUser(
        array $credentials,
        \Institute $institute,
        string $content = ''
    ) {
        $blubber = \BlubberThread::create([
            'context_type' => 'institute',
            'context_id' => $institute->id,
            'content' => $content,
            'user_id' => $credentials['id'],
            'external_contact' => 0,
            'visible_in_stream' => 1,
            'commentable' => 1,
        ]);

        return \BlubberThread::find($blubber->id);
    }

    private function createBlubberComment(array $credentials, \BlubberThread $thread, $content)
    {
        return $this->tester->withPHPLib(
            $credentials,
            fn() => \BlubberComment::find(
                \BlubberComment::create([
                    'thread_id' => $thread->id,
                    'content' => $content,
                    'user_id' => $credentials['id'],
                ])->id
            )
        );
    }

    private function fetchThread(array $credentials, \BlubberThread $resource)
    {
        return $this->tester->callJsonApi(ThreadsShow::class, $credentials, ['id' => $resource->id]);
    }

    private function fetchComments(array $credentials, \BlubberThread $resource)
    {
        return $this->tester->callJsonApi(CommentsByThreadIndex::class, $credentials, ['id' => $resource->id]);
    }

    private function fetchComment(array $credentials, \BlubberComment $resource)
    {
        return $this->tester->callJsonApi(CommentsShow::class, $credentials, ['id' => $resource->id]);
    }

    private function upgradeAndFilterThreads(array $credentials, array $threads)
    {
        return array_filter(
            array_map(fn($thread) => \BlubberThread::upgradeThread($thread), $threads),
            fn($thread) => $thread->isVisibleInStream() && $thread->isReadable($credentials['id'])
        );
    }
}
